##############################################################################
#
#  PROJECT:     Multi Theft Auto v1.0
#  LICENSE:     See LICENSE in the top level directory
#  FILE:        utils/build_gettext_catalog_nsi.py
#  PURPOSE:     Create a template .pot file from a .NSI script
#  DEVELOPERS:  Dan Chowdhury <>
#
#  Multi Theft Auto is available from http://www.multitheftauto.com/
#
#
#  Below script adjusted to fit PhotoQt's needs by: Lukas Spies
#
##############################################################################
import collections
import polib
import datetime
import os
from optparse import OptionParser

parser = OptionParser()
parser.add_option("-v", "--version", dest="version",
                  help="Version to write to the POT file", default="1.x" )

(options, args) = parser.parse_args()

# We only need to either basic or full version, as all strings are the same
options.input = "photoqt_full.nsi"
options.output = "lang_en.po"
options.project = "PhotoQt Windows Installer"
options.lang = "English"

metadata = {
    "Project-Id-Version" : (options.project + " " + options.version).strip(),
    "Report-Msgid-Bugs-To" : "",
    "POT-Creation-Date" : datetime.datetime.now().strftime('%Y-%m-%d %H:%M%z'),
    "PO-Revision-Date" : "YEAR-MO-DA HO:MI+ZONE",
    "Last-Translator" : "FULL NAME <EMAIL@ADDRESS>",
    "Language-Team" : "LANGUAGE <LL@li.org>",
    "Language"  : "en",
    "MIME-Version" : "1.0",
    "Content-Type" : "text/plain; charset=UTF-8",
    "Content-Transfer-Encoding" : "8bit"
}

NSIFilePath = options.input

# Removes trailing \ which marks a new line
def removeEscapedNewLine(line):
    newline = line.rstrip("\n")
    newline = line.rstrip()
    newlen = len(newline);
    if newline.rfind("\\")+1 == len(newline):
        return newline[:newlen-1]
    return line

# Open our source file
NSIWorkingFile = open(NSIFilePath,"r")
NSIWorkingFileDir,NSIFileName = os.path.split(NSIFilePath)
# Create our new .POT file, and give our metadata
poFile = polib.POFile()
poFile.metadata = metadata
# Create a cache of messageValues : [ [fileName1,lineNumber1], [fileName2,lineNumber2]... ]  (The same message could appear on multiple lines)
LangStringCache = collections.OrderedDict()
# Create a cache of messageValues : [ label1, label2 ] (The same message could have multiple NSIS labels)
LangStringLabels = {}

# What we're doing here is looping through each line of our .nsi till we find a LangString of the default language
# Then, we try and grab the line number, the label, and the text
# The text can be multiline, so we have to sometimes continue reading till we reach the end
line=NSIWorkingFile.readline()
lineNo = 1
while line != '':
    commands =  line.split()
    if len(commands) > 3:
        if commands[0] == "LangString" and commands[2].upper() == ("${LANG_%s}"%options.lang).upper():
            label = commands[1]
            value = ""
            # Let's assume it's a one-liner
            start = line.find('"') + 1
            if start:
                end = line.find('"',start)
                if end != -1:
                    value = line[start:end]
                else: # Nope, multiline
                    line = removeEscapedNewLine(line)
                    # Keep reading till we reach the end
                    value = line[start:]
                    line = NSIWorkingFile.readline()
                    lineNo += 1
                    while line != '':
                        line = removeEscapedNewLine(line)
                        end = line.find('"')
                        if end != -1: #If we found the closing character, append
                            value += line[:end].lstrip()
                            break
                        else: #If not, append and continue
                            value += line.lstrip()
                        line=NSIWorkingFile.readline()
                        lineNo += 1

            # Remove whitespace and new lines
            value = value.strip("\t\n")
            value = polib.unescape ( value )
            if not value in LangStringCache:
                LangStringCache[value] = []
            # Note down our file and line number
            LangStringCache[value].append([options.input,lineNo])

            if not value in LangStringLabels:
                LangStringLabels[value] = []
            # Note down our label
            LangStringLabels[value].append(label)

    line=NSIWorkingFile.readline()
    lineNo += 1

# Now, we loop through our cache and build PO entries for each
# We use PO comment field to store our NSIS labels, so we can decode it back later
for msgid,lineOccurances in LangStringCache.iteritems():
    entry = polib.POEntry(
        msgid=msgid,
        msgstr='',
        occurrences=lineOccurances,
        comment=(" ").join(LangStringLabels[msgid])
    )
    poFile.append(entry)


NSIWorkingFile.close()

# Finally, let's generate our POT file
poFile.save(options.output)

print ( "NSI POT Export Operation complete" )



##############################################################################
#
#  PROJECT:     Multi Theft Auto v1.0
#  LICENSE:     See LICENSE in the top level directory
#  FILE:        utils/build_gettext_catalog.py
#  PURPOSE:     Build a multi-language .NSI script based upon input PO files
#  DEVELOPERS:  Dan Chowdhury <>
#
#  Multi Theft Auto is available from http://www.multitheftauto.com/
#
#
#  Below script adjusted to fit PhotoQt's needs by: Lukas Spies
#
##############################################################################
import collections
import os
import polib
from optparse import OptionParser

options_input = "photoqt.nsi"
options_output = "photoqt_multilang.nsi"
options_podir = "" + os.getcwd() + "/lang/" + ""
options_lang = "English"

# Define a dict to convert locale names to language names
localeToName = {
	"af" : "Afrikaans",
	"sq" : "Albanian",
	"ar" : "Arabic",
	"hy" : "Armenian",
	"eu" : "Basque",
	"be" : "Belarusian",
	"bs" : "Bosnian",
	"br" : "Breton",
	"bg" : "Bulgarian",
	"ca" : "Catalan",
	"bem" : "Cibemba",
	"hr" : "Croatian",
	"cs" : "Czech",
	"da" : "Danish",
	"nl" : "Dutch",
	"efi" : "Efik",
	"en" : "English",
	"eo" : "Esperanto",
	"et" : "Estonian",
	"fa" : "Farsi",
	"fi" : "Finnish",
	"fr" : "French",
	"gl" : "Galician",
	"ka" : "Georgian",
	"de" : "German",
	"el" : "Greek",
	"he" : "Hebrew",
	"hi" : "Hindi",
	"hu" : "Hungarian",
	"is" : "Icelandic",
	"ig" : "Igbo",
	"id" : "Indonesian",
	"ga" : "Irish",
	"it" : "Italian",
	"ja" : "Japanese",
	"km" : "Khmer",
	"ko" : "Korean",
	"ku" : "Kurdish",
	"lv" : "Latvian",
	"lt" : "Lithuanian",
	"lb" : "Luxembourgish",
	"mk" : "Macedonian",
	"mg" : "Malagasy",
	"ms" : "Malay",
	"mn" : "Mongolian",
	"nb" : "Norwegian",
	"nn" : "NorwegianNynorsk",
	"ps" : "Pashto",
	"pl" : "Polish",
	"pt" : "Portuguese",
	"pt_BR" : "PortugueseBR",
	"ro" : "Romanian",
	"ru" : "Russian",
	"sr" : "Serbian",
	"sr_sp" : "SerbianLatin",
	"st" : "Sesotho",
	"sn" : "Shona",
	"zh_CN" : "SimpChinese",
	"sk" : "Slovak",
	"sl" : "Slovenian",
	"es" : "Spanish",
	"es_AR" : "SpanishInternational",
	"sw" : "Swahili",
	"sv" : "Swedish",
	"ta" : "Tamil",
	"th" : "Thai",
	"zh_HK" : "TradChinese",
	"tr" : "Turkish",
	"tw" : "Twi",
	"uk" : "Ukrainian",
	"ug" : "Uyghur",
	"uz" : "Uzbek",
	"ca@valencia" : "Valencian",
	"vi" : "Vietnamese",
	"cy" : "Welsh",
	"yo" : "Yoruba",
	"zu" : "Zulu",
}

def escapeNSIS(st):
	return st.replace('\\', r'$\\')\
			.replace('\t', r'$\t')\
			.replace('\r', r'\r')\
			.replace('\n', r'\n')\
			.replace('\"', r'$\"')\
			.replace('$$\\', '$\\')

translationCache = {}

# The purpose of this loop is to go to the podir scanning for PO files for each locale name
# Once we've found a PO file, we use PO lib to read every translated entry
# Using this, for each each language, we store a dict of entries - { nsilabel (comment) : translation (msgstr) }
# For untranslated entries, we use msgid instead of msgstr (i.e. default English string)
for root,dirs,files in os.walk(options_podir):
	for file in files:
		filename,ext = os.path.splitext(file)
		if ext == ".po":
			# Valid locale filename (fr.po, de.po etc)?
			if filename in localeToName:
				language = localeToName[filename]
				translationCache[language] = collections.OrderedDict()

				# Let's add a default LANGUAGE_CODE LangString to be read
				translationCache[language]["LANGUAGE_CODE"] = filename

				po = polib.pofile(os.path.join(root,file))
				for entry in po.translated_entries():
					# Loop through all our labels and add translation (each translation may have multiple labels)
					for label in entry.comment.split():
						translationCache[language][label] = escapeNSIS(entry.msgstr)
				# For untranslated strings, let's add the English entry
				for entry in po.untranslated_entries():
					for label in entry.comment.split():
						print("Warning: Label '%s' for language '%s' remains untranslated"%(label,language))
						translationCache[language][label] = escapeNSIS(entry.msgid)


# Open our source NSI, dump it to a list and close it
NSISourceFile = open(options_input,"r")
NSISourceLines = NSISourceFile.readlines()
NSISourceFile.close()
NSINewLines = []

def tostr(obj):
	if type(obj) == unicode:
		return obj.encode("utf-8")
	else:
		return obj

# Here we scan for ";@INSERT_TRANSLATIONS@" in the NSIS, and add MUI_LANGUAGE macros and LangString's for translation languages
lineNo = 1
for line in NSISourceLines:
	if line.strip().find(";@INSERT_TRANSLATIONS@") == 0:
		for language,translations in translationCache.iteritems():
			count = 0
			# if the language isn't the default, we add our MUI_LANGUAGE macro
			if language.upper() != options_lang.upper():
				NSINewLines.append('!insertmacro MUI_LANGUAGE "' + language + '"\n')
			# For every translation we grabbed from the .po, let's add our LangString
			for label,value in translations.iteritems():
				NSINewLines.append( 'LangString ' + label + ' ${LANG_' + language + '} "' + value + '"\n')
				count += 1
			print ( "%i translations merged for language '%s'"%(count,language) )
	else:
		NSINewLines.append ( line )

# Finally, let's write our new .nsi to the desired target file
NSIWorkingFile = open(options_output,"w")
for newline in NSINewLines:
	NSIWorkingFile.write(newline.encode('utf-8'))
NSIWorkingFile.close()

print("\n\nNSI Localization Operation Complete\n")
